@@ -8,12 +8,11 @@ find . -type f \( -name missing -o -name install-sh -o -name mkinstalldirs \
         -o -name Makefile.in \) | xargs rm -f
 
 rm -f config.sub config.guess aclocal.m4 config.h.in config.rpath
-rm -rf m4
+rm -f m4/libtool.m4 m4/lt~obsolete.m4 m4/ltoptions.m4 m4/ltsugar.m4 m4/ltversion.m4
 cp /usr/share/misc/config.sub .
 cp /usr/share/misc/config.guess .
 cp /usr/share/gettext/config.rpath .
 
 echo Creating autotools files...
-mkdir -p m4
 autoreconf --force --install
 
