@@ -1,5 +1,5 @@
 /* Libvoikko: Library of Finnish language tools
- * Copyright (C) 2009 Harri Pitkänen <hatapitk@iki.fi>
+ * Copyright (C) 2009 - 2010 Harri Pitkänen <hatapitk@iki.fi>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -21,12 +21,12 @@
 #include "grammar/cachesetup.hpp"
 #include "grammar/cache.hpp"
 #include "utils/StringUtils.hpp"
-#include "utils/utils.hpp"
 #include "TrieNode.hpp"
 #include <cstring>
-#include <wchar.h>
+#include <cwchar>
 
 using namespace libvoikko::grammar;
+using namespace libvoikko::utils;
 
 namespace libvoikko { namespace autocorrect {
 
@@ -55,7 +55,7 @@ size_t AutoCorrect::traverse(size_t initial, const wchar_t * str, size_t strlen)
 	return current;
 }
 
-void AutoCorrect::autoCorrect(int handle, const libvoikko::grammar::Sentence * sentence) {
+void AutoCorrect::autoCorrect(voikko_options_t * options, const libvoikko::grammar::Sentence * sentence) {
 	for (size_t i = 0; i + 2 < sentence->tokenCount; i++) {
 		Token t = sentence->tokens[i];
 		if (t.type != TOKEN_WORD) {
@@ -75,9 +75,9 @@ void AutoCorrect::autoCorrect(int handle, const libvoikko::grammar::Sentence * s
 			e->error.error_code = GCERR_INVALID_SPELLING;
 			e->error.startpos = sentence->tokens[i].pos;
 			e->error.errorlen = sentence->tokens[i].tokenlen;
-			e->error.suggestions[0] = voikko_ucs4tocstr(
-			        REPLACEMENTS[NODES[trieNode].replacementIndex], "UTF-8", 0);
-			gc_cache_append_error(handle, e);
+			e->error.suggestions[0] = StringUtils::utf8FromUcs4(
+			        REPLACEMENTS[NODES[trieNode].replacementIndex]);
+			gc_cache_append_error(options, e);
 		}
 		
 		// Is there a second word (in the sentence and in trie)?
@@ -108,9 +108,9 @@ void AutoCorrect::autoCorrect(int handle, const libvoikko::grammar::Sentence * s
 			e->error.startpos = sentence->tokens[i].pos;
 			e->error.errorlen = sentence->tokens[i+2].pos + sentence->tokens[i+2].tokenlen
 			                    - e->error.startpos;
-			e->error.suggestions[0] = voikko_ucs4tocstr(
-			        REPLACEMENTS[NODES[trieNode].replacementIndex], "UTF-8", 0);
-			gc_cache_append_error(handle, e);
+			e->error.suggestions[0] = StringUtils::utf8FromUcs4(
+			        REPLACEMENTS[NODES[trieNode].replacementIndex]);
+			gc_cache_append_error(options, e);
 		}
 	}
 }
