@@ -1,5 +1,5 @@
 /* Libvoikko: Finnish spellchecker and hyphenator library
- * Copyright (C) 2006 - 2009 Harri Pitkänen <hatapitk@iki.fi>,
+ * Copyright (C) 2006 - 2010 Harri Pitkänen <hatapitk@iki.fi>,
  *                           Teemu Likonen <tlikonen@iki.fi>
  *
  * This program is free software; you can redistribute it and/or
@@ -17,11 +17,12 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *********************************************************************************/
 
-#include "voikko_defs.h"
 #include "character/charset.hpp"
+#include "character/SimpleChar.hpp"
 #include <cstdlib>
-#include <wctype.h>
-#include <wchar.h>
+#include <cwchar>
+
+using namespace libvoikko::character;
 
 namespace libvoikko {
 
@@ -37,72 +38,20 @@ char_type get_char_type(wchar_t c) {
 	if (isFinnishQuotationMark(c)) {
 		return CHAR_PUNCTUATION;
 	}
-	if (iswspace(c)) return CHAR_WHITESPACE;
-	if (wcschr(L"aAbBcCdDeEfFgGhHiIjJkKlLmMnNoOpPqQrRsStTuUvVwWxXyYzZ"
-	           L"\u00C0"  /* LATIN CAPITAL LETTER A WITH GRAVE */
-	           L"\u00C1"  /* LATIN CAPITAL LETTER A WITH ACUTE */
-	           L"\u00C2"  /* LATIN CAPITAL LETTER A WITH CIRCUMFLEX */
-	           L"\u00C3"  /* LATIN CAPITAL LETTER A WITH TILDE */
-	           L"\u00C4"  /* LATIN CAPITAL LETTER A WITH DIAERESIS */
-	           L"\u00C5"  /* LATIN CAPITAL LETTER A WITH RING ABOVE */
-	           L"\u00C7"  /* LATIN CAPITAL LETTER C WITH CEDILLA */
-	           L"\u00C8"  /* LATIN CAPITAL LETTER E WITH GRAVE */
-	           L"\u00C9"  /* LATIN CAPITAL LETTER E WITH ACUTE */
-	           L"\u00CA"  /* LATIN CAPITAL LETTER E WITH CIRCUMFLEX */
-	           L"\u00CB"  /* LATIN CAPITAL LETTER E WITH DIAERESIS */
-	           L"\u00CC"  /* LATIN CAPITAL LETTER I WITH GRAVE */
-	           L"\u00CD"  /* LATIN CAPITAL LETTER I WITH ACUTE */
-	           L"\u00CE"  /* LATIN CAPITAL LETTER I WITH CIRCUMFLEX */
-	           L"\u00CF"  /* LATIN CAPITAL LETTER I WITH DIAERESIS */
-	           L"\u00D1"  /* LATIN CAPITAL LETTER N WITH TILDE */
-	           L"\u00D2"  /* LATIN CAPITAL LETTER O WITH GRAVE */
-	           L"\u00D3"  /* LATIN CAPITAL LETTER O WITH ACUTE */
-	           L"\u00D4"  /* LATIN CAPITAL LETTER O WITH CIRCUMFLEX */
-	           L"\u00D5"  /* LATIN CAPITAL LETTER O WITH TILDE */
-	           L"\u00D6"  /* LATIN CAPITAL LETTER O WITH DIAERESIS */
-	           L"\u00D9"  /* LATIN CAPITAL LETTER U WITH GRAVE */
-	           L"\u00DA"  /* LATIN CAPITAL LETTER U WITH ACUTE */
-	           L"\u00DB"  /* LATIN CAPITAL LETTER U WITH CIRCUMFLEX */
-	           L"\u00DC"  /* LATIN CAPITAL LETTER U WITH DIAERESIS */
-	           L"\u00DD"  /* LATIN CAPITAL LETTER Y WITH ACUTE */
-	           L"\u00E0"  /* LATIN SMALL LETTER A WITH GRAVE */
-	           L"\u00E1"  /* LATIN SMALL LETTER A WITH ACUTE */
-	           L"\u00E2"  /* LATIN SMALL LETTER A WITH CIRCUMFLEX */
-	           L"\u00E3"  /* LATIN SMALL LETTER A WITH TILDE */
-	           L"\u00E4"  /* LATIN SMALL LETTER A WITH DIAERESIS */
-	           L"\u00E5"  /* LATIN SMALL LETTER A WITH RING ABOVE */
-	           L"\u00E7"  /* LATIN SMALL LETTER C WITH CEDILLA */
-	           L"\u00E8"  /* LATIN SMALL LETTER E WITH GRAVE */
-	           L"\u00E9"  /* LATIN SMALL LETTER E WITH ACUTE */
-	           L"\u00EA"  /* LATIN SMALL LETTER E WITH CIRCUMFLEX */
-	           L"\u00EB"  /* LATIN SMALL LETTER E WITH DIAERESIS */
-	           L"\u00EC"  /* LATIN SMALL LETTER I WITH GRAVE */
-	           L"\u00ED"  /* LATIN SMALL LETTER I WITH ACUTE */
-	           L"\u00EE"  /* LATIN SMALL LETTER I WITH CIRCUMFLEX */
-	           L"\u00EF"  /* LATIN SMALL LETTER I WITH DIAERESIS */
-	           L"\u00F1"  /* LATIN SMALL LETTER N WITH TILDE */
-	           L"\u00F2"  /* LATIN SMALL LETTER O WITH GRAVE */
-	           L"\u00F3"  /* LATIN SMALL LETTER O WITH ACUTE */
-	           L"\u00F4"  /* LATIN SMALL LETTER O WITH CIRCUMFLEX */
-	           L"\u00F5"  /* LATIN SMALL LETTER O WITH TILDE */
-	           L"\u00F6"  /* LATIN SMALL LETTER O WITH DIAERESIS */
-	           L"\u00F9"  /* LATIN SMALL LETTER U WITH GRAVE */
-	           L"\u00FA"  /* LATIN SMALL LETTER U WITH ACUTE */
-	           L"\u00FB"  /* LATIN SMALL LETTER U WITH CIRCUMFLEX */
-	           L"\u00FC"  /* LATIN SMALL LETTER U WITH DIAERESIS */
-	           L"\u00FD"  /* LATIN SMALL LETTER Y WITH ACUTE */
-	           L"\u00FF"  /* LATIN SMALL LETTER Y WITH DIAERESIS */
-	           L"\u0160"  /* LATIN CAPITAL LETTER S WITH CARON */
-	           L"\u0161"  /* LATIN SMALL LETTER S WITH CARON */
-	           L"\u017D"  /* LATIN CAPITAL LETTER Z WITH CARON */
-	           L"\u017E"  /* LATIN SMALL LETTER Z WITH CARON */
-	           L"\uFB00"  /* LATIN SMALL LIGATURE FF */
-	           L"\uFB01"  /* LATIN SMALL LIGATURE FI */
-	           L"\uFB02"  /* LATIN SMALL LIGATURE FL */
-	           L"\uFB03"  /* LATIN SMALL LIGATURE FFI */
-	           L"\uFB04"  /* LATIN SMALL LIGATURE FFL */
-	           , c)) return CHAR_LETTER;
-	if (wcschr(L"0123456789", c)) return CHAR_DIGIT;
+	if (SimpleChar::isWhitespace(c)) {
+		return CHAR_WHITESPACE;
+	}
+	if ((c >= 0x41 && c <= 0x5A) ||
+	    (c >= 0x61 && c <= 0x7A) ||
+	    (c >= 0xC1 && c <= 0xD6) ||
+	    (c >= 0xD9 && c <= 0xF6) ||
+	    (c >= 0x00F9 && c <= 0x017E) ||
+	    (c >= 0xFB00 && c <= 0xFB04)) {
+		return CHAR_LETTER;
+	}
+	if (wcschr(L"0123456789", c)) {
+		return CHAR_DIGIT;
+	}
 	return CHAR_UNKNOWN;
 }
 
@@ -187,8 +136,8 @@ const wchar_t * CONV_1TO1 =
 const size_t N_1TO2 = 5; /* Number of entries in this table */
 const wchar_t * CONV_1TO2 =
 	/* Letterlike Symbols --> Latin-1 Supplement / Basic Latin */
-	L"\u2103" L"°C"  /* U+2103 DEGREE CELSIUS <--> U+00B0 DEGREE SIGN + U+0043 LATIN CAPITAL LETTER C */
-	L"\u2109" L"°F"  /* U+2109 DEGREE FAHRENHEIT <--> U+00B0 DEGREE SIGN + U+0046 LATIN CAPITAL LETTER F */
+	L"\u2103" L"\u00B0C"  /* U+2103 DEGREE CELSIUS <--> U+00B0 DEGREE SIGN + U+0043 LATIN CAPITAL LETTER C */
+	L"\u2109" L"\u00B0F"  /* U+2109 DEGREE FAHRENHEIT <--> U+00B0 DEGREE SIGN + U+0046 LATIN CAPITAL LETTER F */
 	/* Alphabetic Presentation Forms --> Basic Latin */
 	L"\uFB00" L"ff"  /* LATIN SMALL LIGATURE FF <--> 2 X LATIN SMALL LETTER F */
 	L"\uFB01" L"fi"  /* LATIN SMALL LIGATURE FI <--> LATIN SMALL LETTER F + LATIN SMALL LETTER I */
