@@ -17,10 +17,10 @@
  *********************************************************************************/
 
 #include "grammar/analysis.hpp"
+#include "tokenizer/Tokenizer.hpp"
+#include "sentence/Sentence.hpp"
 #include "utils/StringUtils.hpp"
 #include "utils/utils.hpp"
-#include "setup/setup.hpp"
-#include "morphology/AnalyzerFactory.hpp"
 #include <cstdlib>
 #include <cstring>
 
@@ -31,39 +31,46 @@ namespace libvoikko {
 
 /** Analyze given text token. Token type, length and text must have already
  *  been set. */
-void gc_analyze_token(int /*handle*/, Token * token) {
+static void gc_analyze_token(voikko_options_t * voikkoOptions, Token * token) {
 	token->isValidWord = false;
-	token->firstLetterLcase = false;
 	token->possibleSentenceStart = false;
-	if (token->type != TOKEN_WORD) return;
+	token->isGeographicalNameInGenitive = false;
+	if (token->type != TOKEN_WORD) {
+		token->firstLetterLcase = false;
+		return;
+	}
 	
 	wchar_t * wordBuffer =
 	    utils::StringUtils::stripSpecialCharsForMalaga(token->str,
 	                                                   token->tokenlen);
-	const morphology::Analyzer * analyzer =
-	    morphology::AnalyzerFactory::getAnalyzer();
+	morphology::Analyzer * analyzer = voikkoOptions->morAnalyzer;
 	list<morphology::Analysis *> * analyses = analyzer->analyze(wordBuffer);
 	delete[] wordBuffer;
 	
-	// Check if first letter should be lower case letter
 	list<morphology::Analysis *>::const_iterator it = analyses->begin();
+	token->firstLetterLcase = true;
 	while (it != analyses->end()) {
 		token->isValidWord = true;
 		const wchar_t * structure = (*it)->getValue("STRUCTURE");
 		if (wcslen(structure) < 2 || (structure[1] != L'p' &&
 		    structure[1] != L'q')) {
-			morphology::Analyzer::deleteAnalyses(analyses);
-			return;
+			// Word may start with a capital letter anywhere
+			token->firstLetterLcase = false;
+			const wchar_t * wclass = (*it)->getValue("CLASS");
+			const wchar_t * wcase = (*it)->getValue("SIJAMUOTO");
+			if (wclass && wcscmp(L"paikannimi", wclass) == 0 &&
+			    wcase && wcscmp(L"omanto", wcase) == 0) {
+				token->isGeographicalNameInGenitive = true;
+			}
 		}
 		it++;
 	}
 	morphology::Analyzer::deleteAnalyses(analyses);
-	token->firstLetterLcase = true;
 }
 
 /** Analyze sentence text. Sentence type must be set by the caller. */
-Sentence * gc_analyze_sentence(int handle, const wchar_t * text,
-                                   size_t textlen, size_t sentencepos) {
+static Sentence * gc_analyze_sentence(voikko_options_t * voikkoOptions,
+	          const wchar_t * text, size_t textlen, size_t sentencepos) {
 	Sentence * s = new Sentence;
 	s->pos = sentencepos;
 	size_t tokenlen;
@@ -72,10 +79,10 @@ Sentence * gc_analyze_sentence(int handle, const wchar_t * text,
 	bool next_word_is_possible_sentence_start = false;
 	for (int i = 0; i < Sentence::MAX_TOKENS_IN_SENTENCE; i++) {
 		enum voikko_token_type tt;
-		int ignore_dot_saved = voikko_options.ignore_dot;
-		voikko_options.ignore_dot = 0;
-		tt = voikko_next_token_ucs4(handle, pos, remaining, &tokenlen);
-		voikko_options.ignore_dot = ignore_dot_saved;
+		int ignore_dot_saved = voikkoOptions->ignore_dot;
+		voikkoOptions->ignore_dot = 0;
+		tt = tokenizer::Tokenizer::nextToken(voikkoOptions, pos, remaining, &tokenlen);
+		voikkoOptions->ignore_dot = ignore_dot_saved;
 		if (tt == TOKEN_NONE) return s;
 
 		s->tokens[i].type = tt;
@@ -86,7 +93,7 @@ Sentence * gc_analyze_sentence(int handle, const wchar_t * text,
 		tstr[tokenlen] = L'\0';
 		s->tokens[i].str = tstr;
 		s->tokens[i].pos = sentencepos + (pos - text);
-		gc_analyze_token(handle, s->tokens + i);
+		gc_analyze_token(voikkoOptions, s->tokens + i);
 		
 		if (next_word_is_possible_sentence_start && tt == TOKEN_WORD) {
 			s->tokens[i].possibleSentenceStart = true;
@@ -110,7 +117,7 @@ Sentence * gc_analyze_sentence(int handle, const wchar_t * text,
 }
 
 
-Paragraph * gc_analyze_paragraph(int handle, const wchar_t * text, size_t textlen) {
+Paragraph * gc_analyze_paragraph(voikko_options_t * voikkoOptions, const wchar_t * text, size_t textlen) {
 	Paragraph * p = new Paragraph;
 	const wchar_t * pos = text;
 	size_t remaining = textlen;
@@ -120,14 +127,15 @@ Paragraph * gc_analyze_paragraph(int handle, const wchar_t * text, size_t textle
 		size_t sentencelen = 0;
 		do {
 			size_t sentencelen2;
-			st = voikko_next_sentence_start_ucs4(handle, pos2, remaining,
+			st = sentence::Sentence::next(voikkoOptions, pos2, remaining,
 			                                     &sentencelen2);
 			pos2 += sentencelen2;
 			sentencelen += sentencelen2;
 			remaining -= sentencelen2;
 		} while (st == SENTENCE_POSSIBLE);
 		
-		Sentence * s = gc_analyze_sentence(handle, pos, sentencelen, pos - text);
+		Sentence * s = gc_analyze_sentence(voikkoOptions, pos, sentencelen,
+		                                   pos - text);
 		if (!s) {
 			delete p;
 			return 0;
