@@ -19,32 +19,14 @@
 #ifndef VOIKKO_GRAMMAR_CACHESETUP_H
 #define VOIKKO_GRAMMAR_CACHESETUP_H
 
-#include "voikko_defs.h"
-#include "grammar/CacheEntry.hpp"
+#include "setup/setup.hpp"
 
 namespace libvoikko {
 
-/**
- * Grammar checker cache. Currently the cache can hold only one paragraph.
- */
-class voikko_gc_cache {
-	public:
-	/** Constructs an empty cache */
-	voikko_gc_cache();
-	
-	/** Clears the cache */
-	void clear();
-	
-	/** Null terminated string containing the paragraph text. */
-	wchar_t * paragraph;
-	/** First error in linked list. */
-	grammar::CacheEntry * firstError;
-};
-
 /**
  * Clears grammar checker error cache.
  */
-void gc_clear_cache(int handle);
+void gc_clear_cache(voikko_options_t * handle);
 
 }
 
