@@ -1,5 +1,5 @@
 /* Libvoikko: Finnish spellchecker and hyphenator library
- * Copyright (C) 2008 - 2009 Harri Pitkänen <hatapitk@iki.fi>
+ * Copyright (C) 2008 - 2010 Harri Pitkänen <hatapitk@iki.fi>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -21,17 +21,20 @@
 #include "grammar/cachesetup.hpp"
 #include "grammar/cache.hpp"
 #include "character/charset.hpp"
+#include "character/SimpleChar.hpp"
 #include "setup/setup.hpp"
 #include "utils/utils.hpp"
+#include "utils/StringUtils.hpp"
 #include <cstdlib>
 #include <cstring>
-#include <wctype.h>
 
 using namespace libvoikko::grammar;
+using namespace libvoikko::character;
+using namespace libvoikko::utils;
 
 namespace libvoikko {
 
-void gc_local_punctuation(int handle, const Sentence * sentence) {
+void gc_local_punctuation(voikko_options_t * options, const Sentence * sentence) {
 	CacheEntry * e;
 	for (size_t i = 0; i < sentence->tokenCount; i++) {
 		Token t = sentence->tokens[i];
@@ -44,7 +47,7 @@ void gc_local_punctuation(int handle, const Sentence * sentence) {
 				e->error.errorlen = sentence->tokens[i].tokenlen;
 				e->error.suggestions[0] = new char[2];
 				strcpy(e->error.suggestions[0], " ");
-				gc_cache_append_error(handle, e);
+				gc_cache_append_error(options, e);
 			}
 			else if (i + 1 < sentence->tokenCount) {
 				Token t2 = sentence->tokens[i+1];
@@ -57,7 +60,7 @@ void gc_local_punctuation(int handle, const Sentence * sentence) {
 				e->error.suggestions[0] = new char[2];
 				e->error.suggestions[0][0] = ',';
 				e->error.suggestions[0][1] = L'\0';
-				gc_cache_append_error(handle, e);
+				gc_cache_append_error(options, e);
 			}
 			break;
 		case TOKEN_PUNCTUATION:
@@ -75,7 +78,7 @@ void gc_local_punctuation(int handle, const Sentence * sentence) {
 					e->error.startpos = sentence->tokens[i].pos - 1;
 					e->error.errorlen = 2;
 				}
-				gc_cache_append_error(handle, e);
+				gc_cache_append_error(options, e);
 				continue;
 			}
 			if (t.str[0] == L',' && i + 1 < sentence->tokenCount) {
@@ -88,7 +91,7 @@ void gc_local_punctuation(int handle, const Sentence * sentence) {
 				e->error.errorlen = 2;
 				e->error.suggestions[0] = new char[2];
 				strcpy(e->error.suggestions[0], ",");
-				gc_cache_append_error(handle, e);
+				gc_cache_append_error(options, e);
 			}
 			break;
 		case TOKEN_NONE:
@@ -99,7 +102,7 @@ void gc_local_punctuation(int handle, const Sentence * sentence) {
 	}
 }
 
-void gc_punctuation_of_quotations(int handle, const Sentence * sentence) {
+void gc_punctuation_of_quotations(voikko_options_t * options, const Sentence * sentence) {
 	for (size_t i = 0; i + 2 < sentence->tokenCount; i++) {
 		if (sentence->tokens[i].type != TOKEN_PUNCTUATION) {
 			continue;
@@ -111,8 +114,8 @@ void gc_punctuation_of_quotations(int handle, const Sentence * sentence) {
 			e->error.error_code = GCERR_FOREIGN_QUOTATION_MARK;
 			e->error.startpos = sentence->tokens[i].pos;
 			e->error.errorlen = 1;
-			e->error.suggestions[0] = voikko_ucs4tocstr(L"\u201D", "UTF-8", 1);
-			gc_cache_append_error(handle, e);
+			e->error.suggestions[0] = StringUtils::utf8FromUcs4(L"\u201D", 1);
+			gc_cache_append_error(options, e);
 			return;
 		}
 		if (sentence->tokens[i + 1].type != TOKEN_PUNCTUATION) {
@@ -141,10 +144,10 @@ void gc_punctuation_of_quotations(int handle, const Sentence * sentence) {
 				suggDot[0] = quoteChar;
 				suggDot[1] = L',';
 				suggDot[2] = L'\0';
-				e->error.suggestions[0] = voikko_ucs4tocstr(suggDot, "UTF-8", e->error.errorlen);
+				e->error.suggestions[0] = StringUtils::utf8FromUcs4(suggDot, e->error.errorlen);
 				delete[] suggDot;
 			}
-			gc_cache_append_error(handle, e);
+			gc_cache_append_error(options, e);
 			break;
 		case L'!':
 		case L'?':
@@ -157,111 +160,16 @@ void gc_punctuation_of_quotations(int handle, const Sentence * sentence) {
 				suggOther[0] = (sentence->tokens[i].str[0] == L'!' ? L'!' : L'?');
 				suggOther[1] = quoteChar;
 				suggOther[2] = L'\0';
-				e->error.suggestions[0] = voikko_ucs4tocstr(suggOther, "UTF-8", e->error.errorlen);
+				e->error.suggestions[0] = StringUtils::utf8FromUcs4(suggOther, e->error.errorlen);
 				delete[] suggOther;
 			}
-			gc_cache_append_error(handle, e);
+			gc_cache_append_error(options, e);
 			break;
 		}
 	}
 }
 
-void gc_character_case(int handle, const Sentence * sentence, bool isFirstInParagraph) {
-	// Check if the sentence is written fully in upper case letters.
-	// If it is, no character case errors should be reported.
-	bool onlyUpper = true;
-	for (size_t i = 0; i < sentence->tokenCount; i++) {
-		Token t = sentence->tokens[i];
-		if (t.type != TOKEN_WORD) continue;
-		for (size_t j = 0; j < t.tokenlen; j++) {
-			if (iswlower(t.str[j])) {
-				onlyUpper = false;
-				break;
-			}
-		}
-		if (!onlyUpper) {
-			break;
-		}
-	}
-	if (onlyUpper) {
-		return;
-	}
-	
-	// If the sentence contains a tab character as a word separator, it is
-	// likely that it is actually not a sentence. Maybe it would be best to
-	// disable all checks in that case, but character case checks are more
-	// likely to be wrong than others.
-	// Also the presence of a foreign quotation mark makes it hard to tell
-	// what is the correct capitalization.
-	for (size_t i = 0; i < sentence->tokenCount; i++) {
-		Token t = sentence->tokens[i];
-		if (t.str[0] == L'\u201C') {
-			return;
-		}
-		if (t.type != TOKEN_WHITESPACE) {
-			continue;
-		}
-		for (size_t j = 0; j < t.tokenlen; j++) {
-			if (t.str[j] == L'\t') {
-				return;
-			}
-		}
-	}
-	
-	// Sentences starting with "-" may be list items.
-	bool sentenceStartsWithHyphen =
-		sentence->tokenCount > 0 && sentence->tokens[0].tokenlen == 1 &&
-		sentence->tokens[0].str[0] == '-';
-	
-	bool firstWordSeen = false;
-	bool isInQuotation = false;
-	for (size_t i = 0; i < sentence->tokenCount; i++) {
-		Token t = sentence->tokens[i];
-		if (t.type != TOKEN_WORD) {
-			if (t.tokenlen == 1 && isFinnishQuotationMark(t.str[0])) {
-				// TODO: quotations within quotations do not work this ways
-				isInQuotation = !isInQuotation;
-			}
-			continue;
-		}
-		if (!firstWordSeen) {
-			firstWordSeen = true;
-			bool needCheckingOfFirstUppercase = !sentenceStartsWithHyphen &&
-				(!isFirstInParagraph || !voikko_options.accept_bulleted_lists_in_gc);
-			if (needCheckingOfFirstUppercase && !iswupper(t.str[0]) && !iswdigit(t.str[0])) {
-				CacheEntry * e = new CacheEntry(1);
-				e->error.error_code = GCERR_WRITE_FIRST_UPPERCASE;
-				e->error.startpos = t.pos;
-				e->error.errorlen = t.tokenlen;
-				wchar_t * suggestion = new wchar_t[t.tokenlen];
-				suggestion[0] = towupper(t.str[0]);
-				wcsncpy(suggestion + 1, t.str + 1, t.tokenlen - 1);
-				e->error.suggestions[0] = voikko_ucs4tocstr(suggestion, "UTF-8", t.tokenlen);
-				delete[] suggestion;
-				gc_cache_append_error(handle, e);
-			}
-			continue;
-		}
-		if (isInQuotation) continue;
-		if (!t.isValidWord) continue;
-		if (!t.firstLetterLcase) continue;
-		if (t.possibleSentenceStart) continue;
-		if (t.tokenlen == 1) continue;
-		if (!iswupper(t.str[0])) continue;
-		CacheEntry * e = new CacheEntry(1);
-		e->error.error_code = GCERR_WRITE_FIRST_LOWERCASE;
-		e->error.startpos = t.pos;
-		e->error.errorlen = t.tokenlen;
-		wchar_t * suggestion = new wchar_t[t.tokenlen];
-		suggestion[0] = towlower(t.str[0]);
-		wcsncpy(suggestion + 1, t.str + 1, t.tokenlen - 1);
-		e->error.suggestions[0] = voikko_ucs4tocstr(suggestion, "UTF-8", t.tokenlen);
-		delete[] suggestion;
-		gc_cache_append_error(handle, e);
-	}
-}
-
-void gc_repeating_words(int handle, const Sentence * sentence) {
+void gc_repeating_words(voikko_options_t * options, const Sentence * sentence) {
 	for (size_t i = 0; i + 2 < sentence->tokenCount; i++) {
 		if (sentence->tokens[i].type != TOKEN_WORD) continue;
 		if (sentence->tokens[i + 1].type != TOKEN_WHITESPACE) {
@@ -272,7 +180,12 @@ void gc_repeating_words(int handle, const Sentence * sentence) {
 			i += 2;
 			continue;
 		}
-		if (wcscmp(sentence->tokens[i].str, sentence->tokens[i + 2].str)) {
+		if (!SimpleChar::equalsIgnoreCase(sentence->tokens[i].str, sentence->tokens[i + 2].str)) {
+			i++;
+			continue;
+		}
+		if (wcscmp(sentence->tokens[i].str, L"ollut") == 0) {
+			// "ollut" is a valid word to be repeated, maybe there are others too
 			i++;
 			continue;
 		}
@@ -282,16 +195,16 @@ void gc_repeating_words(int handle, const Sentence * sentence) {
 		e->error.errorlen = sentence->tokens[i].tokenlen +
 		                    sentence->tokens[i + 1].tokenlen +
 		                    sentence->tokens[i + 2].tokenlen;
-		e->error.suggestions[0] = voikko_ucs4tocstr(sentence->tokens[i].str,
-		                          "UTF-8", sentence->tokens[i].tokenlen);
-		gc_cache_append_error(handle, e);
+		e->error.suggestions[0] = StringUtils::utf8FromUcs4(sentence->tokens[i].str,
+		                          sentence->tokens[i].tokenlen);
+		gc_cache_append_error(options, e);
 	}
 }
 
-void gc_end_punctuation(int handle, const Paragraph * paragraph) {
-	if (voikko_options.accept_titles_in_gc && paragraph->sentenceCount == 1) return;
-	if (voikko_options.accept_unfinished_paragraphs_in_gc) return;
-	if (voikko_options.accept_bulleted_lists_in_gc) return;
+void gc_end_punctuation(voikko_options_t * options, const Paragraph * paragraph) {
+	if (options->accept_titles_in_gc && paragraph->sentenceCount == 1) return;
+	if (options->accept_unfinished_paragraphs_in_gc) return;
+	if (options->accept_bulleted_lists_in_gc) return;
 	
 	Sentence * sentence = paragraph->sentences[paragraph->sentenceCount - 1];
 	Token * token = sentence->tokens + (sentence->tokenCount - 1);
@@ -300,7 +213,7 @@ void gc_end_punctuation(int handle, const Paragraph * paragraph) {
 	e->error.error_code = GCERR_TERMINATING_PUNCTUATION_MISSING;
 	e->error.startpos = token->pos;
 	e->error.errorlen = token->tokenlen;
-	gc_cache_append_error(handle, e);
+	gc_cache_append_error(options, e);
 }
 
 }
