@@ -1,5 +1,5 @@
 /* Libvoikko: Finnish spellchecker and hyphenator library
- * Copyright (C) 2008 Harri Pitkänen <hatapitk@iki.fi>
+ * Copyright (C) 2008 - 2010 Harri Pitkänen <hatapitk@iki.fi>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -19,6 +19,8 @@
 #ifndef VOIKKO_GRAMMAR_ERROR_H
 #define VOIKKO_GRAMMAR_ERROR_H
 
+#include "voikko_structs.h"
+
 #define GCERR_INVALID_SPELLING 1
 #define GCERR_EXTRA_WHITESPACE 2
 #define GCERR_SPACE_BEFORE_PUNCTUATION 3
@@ -30,8 +32,7 @@
 #define GCERR_TERMINATING_PUNCTUATION_MISSING 9
 #define GCERR_INVALID_PUNCTUATION_AT_END_OF_QUOTATION 10
 #define GCERR_FOREIGN_QUOTATION_MARK 11
-
-#include "voikko_defs.h"
+#define GCERR_MISPLACED_CLOSING_PARENTHESIS 12
 
 namespace libvoikko {
 
