@@ -1,5 +1,5 @@
 /* Libvoikko: Library of Finnish language tools
- * Copyright (C) 2008 - 2009 Harri Pitkänen <hatapitk@iki.fi>
+ * Copyright (C) 2008 - 2010 Harri Pitkänen <hatapitk@iki.fi>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -24,14 +24,26 @@ namespace libvoikko { namespace setup {
 
 Dictionary::Dictionary() :
 	morPath(),
+	morBackend(),
+	spellBackend(),
+	suggestionBackend(),
+	hyphenatorBackend(),
 	variant(),
 	description(),
 	isDefaultDict(false) {
 }
 
-Dictionary::Dictionary(const string & morPath, const string & variant,
+Dictionary::Dictionary(const string & morPath, const string & morBackend,
+                       const string & spellBackend,
+                       const string & suggestionBackend,
+                       const string & hyphenatorBackend,
+                       const string & variant,
                        const string & description) :
 	morPath(morPath),
+	morBackend(morBackend),
+	spellBackend(spellBackend),
+	suggestionBackend(suggestionBackend),
+	hyphenatorBackend(hyphenatorBackend),
 	variant(variant),
 	description(description),
 	isDefaultDict(false) {
@@ -39,6 +51,10 @@ Dictionary::Dictionary(const string & morPath, const string & variant,
 
 Dictionary::Dictionary(const Dictionary & dictionary) :
 	morPath(dictionary.morPath),
+	morBackend(dictionary.morBackend),
+	spellBackend(dictionary.spellBackend),
+	suggestionBackend(dictionary.suggestionBackend),
+	hyphenatorBackend(dictionary.hyphenatorBackend),
 	variant(dictionary.variant),
 	description(dictionary.description),
 	isDefaultDict(dictionary.isDefaultDict) {
@@ -48,6 +64,22 @@ const string & Dictionary::getMorPath() const {
 	return morPath;
 }
 
+const string & Dictionary::getMorBackend() const {
+	return morBackend;
+}
+
+const string & Dictionary::getSpellBackend() const {
+	return spellBackend;
+}
+
+const string & Dictionary::getSuggestionBackend() const {
+	return suggestionBackend;
+}
+
+const string & Dictionary::getHyphenatorBackend() const {
+	return hyphenatorBackend;
+}
+
 const string & Dictionary::getVariant() const {
 	return variant;
 }
