@@ -1,5 +1,5 @@
 /* Libvoikko: Library of Finnish language tools
- * Copyright (C) 2008 - 2009 Harri Pitkänen <hatapitk@iki.fi>
+ * Copyright (C) 2008 - 2010 Harri Pitkänen <hatapitk@iki.fi>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,7 +17,7 @@
  *********************************************************************************/
 
 #include "setup/DictionaryLoader.hpp"
-#include "config.h"
+#include "porting.h"
 #include <string>
 #include <fstream>
 #include <cstdlib>
@@ -27,7 +27,6 @@
 # include <pwd.h>
 # include <dirent.h>
 #endif
-#include "morphology/malaga/malaga.hpp"
 
 #define VOIKKO_DICTIONARY_FILE "voikko-fi_FI.pro"
 #define VOIKKO_DICTIONARY_VERSION "2"
@@ -39,10 +38,52 @@
 #endif
 
 using namespace std;
-using namespace libvoikko::morphology::malaga;
 
 namespace libvoikko { namespace setup {
 
+struct LanguageTag {
+	string language;
+	string privateUse;
+};
+
+static void tagToCanonicalForm(string & languageTag) {
+	for (size_t i = 0; i < languageTag.size(); ++i) {
+		char current = languageTag.at(i);
+		if (current >= 65 && current <= 90) {
+			languageTag[i] = current + 32;
+		}
+	}
+}
+
+static LanguageTag parseFromBCP47(const string & language) {
+	// TODO: this parsing algorithm is incomplete
+	LanguageTag tag;
+	if (language.size() < 2) {
+		return tag;
+	}
+	
+	string canonicalLanguage = language;
+	tagToCanonicalForm(canonicalLanguage);
+	
+	if (canonicalLanguage.compare(0, 2, "fi") == 0) {
+		tag.language = "fi";
+	} else {
+		return tag;
+	}
+	
+	size_t privateUseStart = canonicalLanguage.find("-x-");
+	if (privateUseStart != string::npos) {
+		string privateUse = canonicalLanguage.substr(privateUseStart + 3);
+		for (size_t hyphenPos = privateUse.find("-"); hyphenPos != string::npos;
+		            hyphenPos = privateUse.find("-")) {
+			privateUse.erase(hyphenPos, 1);
+		}
+		tag.privateUse = privateUse;
+	}
+	
+	return tag;
+}
+
 list<Dictionary> DictionaryLoader::findAllAvailable() {
 	return findAllAvailable(string());
 }
@@ -55,12 +96,12 @@ list<Dictionary> DictionaryLoader::findAllAvailable(const std::string & path) {
 	}
 	
 	map<string, Dictionary> dictMap;
-	for (list<string>::iterator i = locations.begin(); i != locations.end(); i++) {
+	for (list<string>::iterator i = locations.begin(); i != locations.end(); ++i) {
 		addVariantsFromPath(*i, dictMap);
 	}
 	
 	list<Dictionary> dicts;
-	for (map< string, Dictionary >::iterator i = dictMap.begin(); i != dictMap.end(); i++) {
+	for (map< string, Dictionary >::iterator i = dictMap.begin(); i != dictMap.end(); ++i) {
 		if (i->second.isDefault()) {
 			dicts.push_front(i->second);
 		}
@@ -74,34 +115,37 @@ list<Dictionary> DictionaryLoader::findAllAvailable(const std::string & path) {
 	return dicts;
 }
 
-Dictionary DictionaryLoader::load(const string & variant) throw(DictionaryException) {
-	return load(variant, string());
+Dictionary DictionaryLoader::load(const string & language) throw(DictionaryException) {
+	return load(language, string());
 }
 
-Dictionary DictionaryLoader::load(const string & variant, const string & path)
+Dictionary DictionaryLoader::load(const string & language, const string & path)
 		throw(DictionaryException) {
+	LanguageTag tag = parseFromBCP47(language);
+	if (tag.language != "fi") {
+		throw DictionaryException("Requested language is not supported");
+	}
+	
 	list<Dictionary> dicts = findAllAvailable(path);
-	if (dicts.size() == 0) {
+	if (dicts.empty()) {
 		throw DictionaryException("No valid dictionaries were found");
 	}
 	
-	string finalVariant(variant);
-	if (variant.empty() || variant == "default" || variant == "fi_FI") {
+	string finalVariant(tag.privateUse);
+	if (finalVariant.empty() || finalVariant == "default" || finalVariant == "fi_FI") {
 		// Use dictionary specified by environment variable VOIKKO_DICTIONARY_PATH
-		// FIXME: thread safety
+		// XXX: Not actually thread safe but will most probably work
 		char * dict_from_env = getenv("VOIKKO_DICTIONARY");
 		if (dict_from_env) {
 			finalVariant = string(dict_from_env);
 		}
 		else {
 			// No dictionary specified anywhere, so take the one with highest priority.
-			loadDictionary(dicts.front());
 			return dicts.front();
 		}
 	}
-	for (list<Dictionary>::iterator i = dicts.begin(); i != dicts.end(); i++) {
+	for (list<Dictionary>::iterator i = dicts.begin(); i != dicts.end(); ++i) {
 		if ((*i).getVariant() == finalVariant) {
-			loadDictionary(*i);
 			return *i;
 		}
 	}
@@ -179,24 +223,42 @@ Dictionary DictionaryLoader::dictionaryFromPath(const string & path) {
 	
 	string variant;
 	string description;
+	string morBackend = "malaga";
+	string spellBackend = "AnalyzerToSpellerAdapter(currentAnalyzer)";
+	string suggestionBackend = "FinnishSuggestionStrategy(currentAnalyzer)";
+	string hyphenatorBackend = "AnalyzerToFinnishHyphenatorAdapter(currentAnalyzer)";
 	while (file.good()) {
 		getline(file, line);
 		if (line.find("info: Language-Variant: ") == 0) {
 			variant = line.substr(24);
+			tagToCanonicalForm(variant);
 		}
 		else if (line.find("info: Description: ") == 0) {
 			description = line.substr(19);
 		}
+		else if (line.find("info: Morphology-Backend: ") == 0) {
+			morBackend = line.substr(26);
+		}
+		else if (line.find("info: Speller-Backend: ") == 0) {
+			spellBackend = line.substr(23);
+		}
+		else if (line.find("info: Suggestion-Backend: ") == 0) {
+			suggestionBackend = line.substr(26);
+		}
+		else if (line.find("info: Hyphenator-Backend: ") == 0) {
+			hyphenatorBackend = line.substr(26);
+		}
 	}
 	file.close();
-	return Dictionary(path, variant, description);
+	return Dictionary(path, morBackend, spellBackend, suggestionBackend,
+	                  hyphenatorBackend,variant, description);
 }
 
 list<string> DictionaryLoader::getDefaultLocations() {
 	list<string> locations;
 	
 	/* Path specified by environment variable VOIKKO_DICTIONARY_PATH */
-	// FIXME: thread safety
+	// XXX: Not actually thread safe but will most probably work
 	char * path_from_env = getenv("VOIKKO_DICTIONARY_PATH");
 	if (path_from_env) {
 		locations.push_back(string(path_from_env));
@@ -260,7 +322,7 @@ list<string> DictionaryLoader::getDefaultLocations() {
 }
 
 bool DictionaryLoader::hasDefault(map<string, Dictionary> & variants) {
-	for (map<string, Dictionary>::iterator i = variants.begin(); i != variants.end(); i++) {
+	for (map<string, Dictionary>::iterator i = variants.begin(); i != variants.end(); ++i) {
 		if (i->second.isDefault()) {
 			return true;
 		}
@@ -268,15 +330,4 @@ bool DictionaryLoader::hasDefault(map<string, Dictionary> & variants) {
 	return false;
 }
 
-void DictionaryLoader::loadDictionary(const Dictionary & dictionary)
-		throw(DictionaryException) {
-	string projectFile(dictionary.getMorPath());
-	projectFile.append("/");
-	projectFile.append(VOIKKO_DICTIONARY_FILE);
-	init_libmalaga(projectFile.c_str());
-	if (malaga_error) {
-		throw DictionaryException(malaga_error);
-	}
-}
-
 } }
