@@ -1,5 +1,5 @@
 /* Libvoikko: Finnish spellchecker and hyphenator library
- * Copyright (C) 2006 - 2009 Harri Pitkänen <hatapitk@iki.fi>
+ * Copyright (C) 2006 - 2010 Harri Pitkänen <hatapitk@iki.fi>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -19,17 +19,16 @@
 #ifndef VOIKKO_SETUP_SETUP_H
 #define VOIKKO_SETUP_SETUP_H
 
-#include "grammar/cachesetup.hpp"
-#ifdef HAVE_ICONV
-  #include <iconv.h>
-#endif
-#include <wchar.h>
+#include "morphology/Analyzer.hpp"
+#include "spellchecker/Speller.hpp"
+#include "spellchecker/SpellerCache.hpp"
+#include "spellchecker/suggestion/SuggestionGenerator.hpp"
+#include "hyphenator/Hyphenator.hpp"
+#include "setup/Dictionary.hpp"
+#include "grammar/GcCache.hpp"
 
 namespace libvoikko {
 
-/** Suggestion types */
-enum suggtype {ST_STD, ST_OCR};
-
 typedef struct {
 	int ignore_dot;
 	int ignore_numbers;
@@ -37,33 +36,20 @@ typedef struct {
 	int ignore_nonwords;
 	int accept_first_uppercase;
 	int accept_all_uppercase;
-	int no_ugly_hyphenation;
 	int accept_extra_hyphens;
 	int accept_missing_hyphens;
-	int intersect_compound_level;
-	size_t min_hyphenated_word_length;
 	int accept_titles_in_gc;
 	int accept_unfinished_paragraphs_in_gc;
 	int accept_bulleted_lists_in_gc;
-	int hyphenate_unknown_words;
-	const char * encoding;
-	#ifdef HAVE_ICONV
-	iconv_t iconv_ucs4_utf8;
-	iconv_t iconv_utf8_ucs4;
-	iconv_t iconv_ucs4_ext;
-	iconv_t iconv_ext_ucs4;
-	#endif
-	wchar_t * cache;
-	char * cache_meta;
-	int cache_size;
-	enum suggtype suggestion_type;
-	voikko_gc_cache gc_cache;
+	grammar::GcCache gc_cache;
+	morphology::Analyzer * morAnalyzer;
+	spellchecker::Speller * speller;
+	spellchecker::SpellerCache * spellerCache;
+	spellchecker::suggestion::SuggestionGenerator * suggestionGenerator;
+	hyphenator::Hyphenator * hyphenator;
+	setup::Dictionary dictionary;
 } voikko_options_t;
 
-extern voikko_options_t voikko_options;
-
-extern int voikko_handle_count;
-
 }
 
 #endif
