@@ -16,56 +16,31 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *********************************************************************************/
 
-#include "voikko_defs.h"
-#include "setup/setup.hpp"
 #include "utils/utils.hpp"
+#include "utils/StringUtils.hpp"
 #include "character/charset.hpp"
+#include "character/SimpleChar.hpp"
 #include "spellchecker/spell.hpp"
-#include "morphology/AnalyzerFactory.hpp"
 #include "spellchecker/SpellUtils.hpp"
+#include "porting.h"
 #include <cstdlib>
 #include <cstring>
-#include <wchar.h>
-#include <wctype.h>
+#include <cwchar>
+#include <cassert>
 
 using namespace libvoikko::morphology;
 using namespace libvoikko::spellchecker;
+using namespace libvoikko::character;
 using namespace std;
 
 namespace libvoikko {
 
-static spellresult bestAnalysis(const wchar_t * word, size_t len) {
-	const Analyzer * analyzer = AnalyzerFactory::getAnalyzer();
-	list<Analysis *> * analyses = analyzer->analyze(word, len);
-	
-	if (analyses->empty()) {
-		Analyzer::deleteAnalyses(analyses);
-		return SPELL_FAILED;
-	}
-	
-	spellresult best_result = SPELL_FAILED;
-	list<Analysis *>::const_iterator it = analyses->begin();
-	while (it != analyses->end()) {
-		const wchar_t * structure = (*it)->getValue("STRUCTURE");
-		spellresult result = SpellUtils::matchWordAndAnalysis(word, len, structure);
-		if (best_result == SPELL_FAILED || best_result > result) {
-			best_result = result;
-		}
-		if (best_result == SPELL_OK) {
-			break;
-		}
-		it++;
-	}
-	Analyzer::deleteAnalyses(analyses);
-	
-	return best_result;
-}
-
-spellresult voikko_do_spell(const wchar_t * word, size_t len) {
+spellresult voikko_do_spell(voikko_options_t * voikkoOptions,
+	                   const wchar_t * word, size_t len) {
 	spellresult result_with_border = SPELL_FAILED;
 	spellresult result_without_border = SPELL_FAILED;
 	
-	enum spellresult result = bestAnalysis(word, len);
+	spellresult result = voikkoOptions->speller->spell(word, len);
 	const wchar_t * hyphen_pos;
 	if (result != SPELL_OK && len > 3) {
 		hyphen_pos = wmemchr(word + 1, L'-', len - 2);
@@ -81,11 +56,11 @@ spellresult voikko_do_spell(const wchar_t * word, size_t len) {
 		wcsncpy(buffer + leading_len, hyphen_pos + 1, len - leading_len - 1);
 		buffer[len - 1] = L'\0';
 		
-		if (voikko_options.accept_extra_hyphens && leading_len > 1 &&
+		if (voikkoOptions->accept_extra_hyphens && leading_len > 1 &&
 		    buffer[leading_len] != L'-') {
 			/* All hyphens are optional */
 			/* FIXME: deep recursion */
-			spellresult spres = voikko_do_spell(buffer, len - 1);
+			spellresult spres = voikko_do_spell(voikkoOptions, buffer, len - 1);
 			if (spres == SPELL_OK) {
 				delete[] buffer;
 				return spres;
@@ -94,13 +69,13 @@ spellresult voikko_do_spell(const wchar_t * word, size_t len) {
 		
 		/* Leading part ends with the same VC pair as the trailing part starts ('pop-opisto') */
 		if (leading_len >= 2 && len - leading_len >= 3) {
-			wint_t vctest1 = towlower(word[leading_len - 2]);
-			wint_t vctest2 = towlower(word[leading_len - 1]);
+			wchar_t vctest1 = SimpleChar::lower(word[leading_len - 2]);
+			wchar_t vctest2 = SimpleChar::lower(word[leading_len - 1]);
 			if (wcschr(VOIKKO_VOWELS, vctest1) &&
 			    wcschr(VOIKKO_CONSONANTS, vctest2) &&
-			    towlower(word[leading_len + 1]) == vctest1 &&
-			    towlower(word[leading_len + 2]) == vctest2) {
-				spellresult spres = bestAnalysis(buffer, len - 1);
+			    SimpleChar::lower(word[leading_len + 1]) == vctest1 &&
+			    SimpleChar::lower(word[leading_len + 2]) == vctest2) {
+				spellresult spres = voikkoOptions->speller->spell(buffer, len - 1);
 				if (spres != SPELL_FAILED && (result == SPELL_FAILED || result > spres)) {
 					delete[] buffer;
 					return spres;
@@ -109,8 +84,7 @@ spellresult voikko_do_spell(const wchar_t * word, size_t len) {
 		}
 		
 		/* Ambiguous compound ('syy-silta', 'syys-ilta') */
-		const Analyzer * analyzer = AnalyzerFactory::getAnalyzer();
-		list<Analysis *> * analyses = analyzer->analyze(buffer);
+		list<Analysis *> * analyses = voikkoOptions->morAnalyzer->analyze(buffer);
 		
 		if (analyses->empty()) {
 			Analyzer::deleteAnalyses(analyses);
@@ -157,8 +131,15 @@ spellresult voikko_do_spell(const wchar_t * word, size_t len) {
 	return result;
 }
 
-spellresult voikko_do_spell_ignore_hyphens(const wchar_t * word, size_t len) {
-	spellresult spres = voikko_do_spell(word, len);
+/** Checks the spelling of given word. Missing hyphens at the start or end of the
+ * word are ignored.
+ * @param word word to check (does not need to be null terminated)
+ * @param len length of the word to check
+ * @return spelling result
+ */
+static spellresult voikko_do_spell_ignore_hyphens(voikko_options_t * voikkoOptions,
+	                                   const wchar_t * word, size_t len) {
+	spellresult spres = voikko_do_spell(voikkoOptions, word, len);
 	if (spres != SPELL_FAILED) return spres;
 	
 	// Hyphens were already present, so we cannot do anything more
@@ -178,126 +159,110 @@ spellresult voikko_do_spell_ignore_hyphens(const wchar_t * word, size_t len) {
 			newlen++;
 		}
 	}
-	spres = voikko_do_spell(buffer, newlen);
+	spres = voikko_do_spell(voikkoOptions, buffer, newlen);
 	delete[] buffer;
 	return spres;
 }
 
-/* A small result cache:
- * word_length order elements wchars offset
- * 1           4     16       16     0
- * 2           5     32       64     16
- * 3           6     64       192    80
- * 4           7     128      512    272
- * 5           7     128      640    784
- * 6           7     128      768    1424
- * 7           7     128      896    2192
- * 8           7     128      1024   3088
- * 9           7     128      1152   4112
- * 10          7     128      1280   5264
- * total size      sizeof(wchar_t) * 6544
+
+/** Checks the spelling of given word and uses cache if possible
+ * @param word word to check. Word does not need to be null terminated and it must
+ *             not contain upper case letters
+ * @param len length of the word to check
+ * @return spelling result
  */
-
-const int VOIKKO_HASH_ORDERS[]   =  {0, 3+0,  5+0,  6+0,   7+0,   7+0,    7+0,    7+0,    7+0,    7+0,    7+0};
-const int VOIKKO_CACHE_OFFSETS[] =  {0,   0, 1*16, 1*80, 1*272, 1*784, 1*1424, 1*2192, 1*3088, 1*4112, 1*5264};
-const int VOIKKO_META_OFFSETS[]  =  {0,   0, 1*16, 1*48, 1*112, 1*240,  1*368,  1*496,  1*624,  1*752,  1*880};
-
-spellresult voikko_cached_spell(const wchar_t * buffer, size_t len) {
-	int sparam = voikko_options.cache_size;
-	if (voikko_options.cache && len <= 10) { /* check cache */
-		int hashcode = voikko_hash(buffer, len, VOIKKO_HASH_ORDERS[len] + sparam);
-		int cache_offset = (VOIKKO_CACHE_OFFSETS[len] << sparam) + hashcode * static_cast<int>(len);
-		int meta_offset = (VOIKKO_META_OFFSETS[len] << sparam) + hashcode;
-		if (wcsncmp(voikko_options.cache + cache_offset, buffer, len) == 0) {
-			/* DEBUG: printf("Cache hit: '%ls'\n", buffer);*/
-			if (voikko_options.cache_meta[meta_offset] == 'i') return SPELL_CAP_FIRST;
-			else return SPELL_OK;
+static spellresult voikko_cached_spell(voikko_options_t * voikkoOptions, const wchar_t * buffer, size_t len) {
+	SpellerCache * cache = voikkoOptions->spellerCache;
+	if (cache) {
+		if (cache->isInCache(buffer, len)) {
+			/* is in cache */
+			return cache->getSpellResult(buffer, len);
 		}
 		/* not in cache */
 		spellresult result;
-		if (voikko_options.accept_missing_hyphens) {
-			result = voikko_do_spell_ignore_hyphens(buffer, len);
+		if (voikkoOptions->accept_missing_hyphens) {
+			result = voikko_do_spell_ignore_hyphens(voikkoOptions, buffer, len);
 		}
 		else {
-			result = voikko_do_spell(buffer, len);
-		}
-		if (result == SPELL_OK || result == SPELL_CAP_FIRST) {
-			wcsncpy(voikko_options.cache + cache_offset, buffer, len);
-			voikko_options.cache_meta[meta_offset] = (result == SPELL_OK) ? 'p' : 'i';
+			result = voikko_do_spell(voikkoOptions, buffer, len);
 		}
+		cache->setSpellResult(buffer, len, result);
 		return result;
 	}
 	/* no cache available */
-	if (voikko_options.accept_missing_hyphens) {
-		return voikko_do_spell_ignore_hyphens(buffer, len);
+	if (voikkoOptions->accept_missing_hyphens) {
+		return voikko_do_spell_ignore_hyphens(voikkoOptions, buffer, len);
 	}
 	else {
-		return voikko_do_spell(buffer, len);
+		return voikko_do_spell(voikkoOptions, buffer, len);
 	}
 }
 
 
-VOIKKOEXPORT int voikko_spell_ucs4(int /*handle*/, const wchar_t * word) {
+VOIKKOEXPORT int voikkoSpellUcs4(voikko_options_t * voikkoOptions, const wchar_t * word) {
 	size_t nchars = wcslen(word);
 	int result;
 	spellresult sres;
-	if (nchars == 0) return VOIKKO_SPELL_OK;
-	if (nchars > LIBVOIKKO_MAX_WORD_CHARS) return VOIKKO_INTERNAL_ERROR;
-	ENTER_V
+	if (nchars == 0) {
+		return VOIKKO_SPELL_OK;
+	}
+	if (nchars > LIBVOIKKO_MAX_WORD_CHARS) {
+		return VOIKKO_INTERNAL_ERROR;
+	}
 	
 	wchar_t * nword = voikko_normalise(word, nchars);
 	if (nword == 0) {
-		EXIT_V
 		return VOIKKO_INTERNAL_ERROR;
 	}
 	nchars = wcslen(nword);
 	
-	if (voikko_options.ignore_numbers) {
+	if (voikkoOptions->ignore_numbers) {
 		for (size_t i = 0; i < nchars; i++) {
-			if (iswdigit(nword[i])) {
+			if (SimpleChar::isDigit(nword[i])) {
 				delete[] nword;
-				EXIT_V
 				return VOIKKO_SPELL_OK;
 			}
 		}
 	}
 	casetype caps = voikko_casetype(nword, nchars);
-	if ((voikko_options.ignore_uppercase && caps == CT_ALL_UPPER) ||
-	    (voikko_options.ignore_nonwords && voikko_is_nonword(nword, nchars))) {
+	if ((voikkoOptions->ignore_uppercase && caps == CT_ALL_UPPER) ||
+	    (voikkoOptions->ignore_nonwords && voikko_is_nonword(nword, nchars))) {
 		delete[] nword;
-		EXIT_V
 		return VOIKKO_SPELL_OK;
 	}
 	
 	wchar_t * buffer = new wchar_t[nchars + 1];
 
 	for (size_t i = 0; i < nchars; i++) {
-		buffer[i] = towlower(nword[i]);
+		buffer[i] = SimpleChar::lower(nword[i]);
 	}
 	buffer[nchars] = L'\0';
 	
 	int dot_index;
-	if (voikko_options.ignore_dot && buffer[nchars - 1] == L'.') {
+	size_t realChars;
+	if (voikkoOptions->ignore_dot && buffer[nchars - 1] == L'.') {
 		dot_index = static_cast<int>(nchars - 1);
 		buffer[dot_index] = L'\0';
+		realChars = nchars - 1;
 	}
 	else {
 		dot_index = -1;
+		realChars = nchars;
 	}
 	
 	/* Check words that require exact captialisation */
 	if (caps == CT_COMPLEX || caps == CT_NO_LETTERS ||
-	    (caps == CT_ALL_UPPER && !voikko_options.accept_all_uppercase)) {
+	    (caps == CT_ALL_UPPER && !voikkoOptions->accept_all_uppercase)) {
 		wcsncpy(buffer, nword, nchars);
-		buffer[0] = towlower(buffer[0]);
-		if (voikko_options.accept_missing_hyphens) {
-			sres = voikko_do_spell_ignore_hyphens(buffer, nchars);
+		buffer[0] = SimpleChar::lower(buffer[0]);
+		if (voikkoOptions->accept_missing_hyphens) {
+			sres = voikko_do_spell_ignore_hyphens(voikkoOptions, buffer, nchars);
 		}
 		else {
-			sres = voikko_do_spell(buffer, nchars);
+			sres = voikko_do_spell(voikkoOptions, buffer, nchars);
 		}
 		if (sres == SPELL_OK ||
-		    (sres == SPELL_CAP_FIRST && voikko_options.accept_first_uppercase && iswupper(nword[0]))) {
+		    (sres == SPELL_CAP_FIRST && voikkoOptions->accept_first_uppercase && SimpleChar::isUpper(nword[0]))) {
 			result = VOIKKO_SPELL_OK;
 		}
 		else {
@@ -305,20 +270,19 @@ VOIKKOEXPORT int voikko_spell_ucs4(int /*handle*/, const wchar_t * word) {
 		}
 		if (result == VOIKKO_SPELL_FAILED && dot_index != -1) { /* remove dot */
 			buffer[dot_index] = L'\0';
-			if (voikko_options.accept_missing_hyphens) {
-				sres = voikko_do_spell_ignore_hyphens(buffer, nchars);
+			if (voikkoOptions->accept_missing_hyphens) {
+				sres = voikko_do_spell_ignore_hyphens(voikkoOptions, buffer, realChars);
 			}
 			else {
-				sres = voikko_do_spell(buffer, nchars);
+				sres = voikko_do_spell(voikkoOptions, buffer, realChars);
 			}
 			if (sres == SPELL_OK ||
-			    (sres == SPELL_CAP_FIRST && voikko_options.accept_first_uppercase && iswupper(nword[0]))) {
+			    (sres == SPELL_CAP_FIRST && voikkoOptions->accept_first_uppercase && SimpleChar::isUpper(nword[0]))) {
 				result = VOIKKO_SPELL_OK;
 			}
 		}
 		delete[] nword;
 		delete[] buffer;
-		EXIT_V
 		return result;
 	}
 	
@@ -326,18 +290,18 @@ VOIKKOEXPORT int voikko_spell_ucs4(int /*handle*/, const wchar_t * word) {
 	/* Check without trailing dot */
 	switch (caps) {
 		case CT_ALL_LOWER:
-			sres = voikko_cached_spell(buffer, nchars);
+			sres = voikko_cached_spell(voikkoOptions, buffer, realChars);
 			result = (sres == SPELL_OK) ? VOIKKO_SPELL_OK : VOIKKO_SPELL_FAILED;
 			break;
 		case CT_FIRST_UPPER:
-			sres = voikko_cached_spell(buffer, nchars);
-			if ((sres == SPELL_OK && voikko_options.accept_first_uppercase) || sres == SPELL_CAP_FIRST)
+			sres = voikko_cached_spell(voikkoOptions, buffer, realChars);
+			if ((sres == SPELL_OK && voikkoOptions->accept_first_uppercase) || sres == SPELL_CAP_FIRST)
 				result = VOIKKO_SPELL_OK;
 			else result = VOIKKO_SPELL_FAILED;
 			break;
 		case CT_ALL_UPPER:
-			assert(voikko_options.accept_all_uppercase);
-			sres = voikko_cached_spell(buffer, nchars);
+			assert(voikkoOptions->accept_all_uppercase);
+			sres = voikko_cached_spell(voikkoOptions, buffer, realChars);
 			result = (sres == SPELL_FAILED) ? VOIKKO_SPELL_FAILED : VOIKKO_SPELL_OK;
 			break;
 		default: /* should not happen */
@@ -354,18 +318,18 @@ VOIKKOEXPORT int voikko_spell_ucs4(int /*handle*/, const wchar_t * word) {
 		buffer[dot_index] = L'.';
 		switch (caps) {
 			case CT_ALL_LOWER:
-				sres = voikko_cached_spell(buffer, nchars);
+				sres = voikko_cached_spell(voikkoOptions, buffer, nchars);
 				result = (sres == SPELL_OK) ? VOIKKO_SPELL_OK : VOIKKO_SPELL_FAILED;
 				break;
 			case CT_FIRST_UPPER:
-				sres = voikko_cached_spell(buffer, nchars);
-				if ((sres == SPELL_OK && voikko_options.accept_first_uppercase) || sres == SPELL_CAP_FIRST)
+				sres = voikko_cached_spell(voikkoOptions, buffer, nchars);
+				if ((sres == SPELL_OK && voikkoOptions->accept_first_uppercase) || sres == SPELL_CAP_FIRST)
 					result = VOIKKO_SPELL_OK;
 				else result = VOIKKO_SPELL_FAILED;
 				break;
 			case CT_ALL_UPPER:
-				assert(voikko_options.accept_all_uppercase);
-				sres = voikko_cached_spell(buffer, nchars);
+				assert(voikkoOptions->accept_all_uppercase);
+				sres = voikko_cached_spell(voikkoOptions, buffer, nchars);
 				result = (sres == SPELL_FAILED) ? VOIKKO_SPELL_FAILED : VOIKKO_SPELL_OK;
 			default: /* should not happen */
 				result = VOIKKO_INTERNAL_ERROR;
@@ -376,13 +340,19 @@ VOIKKOEXPORT int voikko_spell_ucs4(int /*handle*/, const wchar_t * word) {
 	return result;
 }
 
-VOIKKOEXPORT int voikko_spell_cstr(int handle, const char * word) {
-	if (word == 0 || word[0] == '\0') return VOIKKO_SPELL_OK;
+VOIKKOEXPORT int voikkoSpellCstr(voikko_options_t * handle, const char * word) {
+	if (word == 0 || word[0] == '\0') {
+		return VOIKKO_SPELL_OK;
+	}
 	size_t len = strlen(word);
-	if (len > LIBVOIKKO_MAX_WORD_CHARS) return 0;
-	wchar_t * word_ucs4 = voikko_cstrtoucs4(word, voikko_options.encoding, len);
-	if (word_ucs4 == 0) return VOIKKO_CHARSET_CONVERSION_FAILED;
-	int result = voikko_spell_ucs4(handle, word_ucs4);
+	if (len > LIBVOIKKO_MAX_WORD_CHARS) {
+		return 0;
+	}
+	wchar_t * word_ucs4 = utils::StringUtils::ucs4FromUtf8(word, len);
+	if (word_ucs4 == 0) {
+		return VOIKKO_CHARSET_CONVERSION_FAILED;
+	}
+	int result = voikkoSpellUcs4(handle, word_ucs4);
 	delete[] word_ucs4;
 	return result;
 }
