@@ -1,5 +1,5 @@
 /* Libvoikko: Finnish spellchecker and hyphenator library
- * Copyright (C) 2009 Harri Pitkänen <hatapitk@iki.fi>
+ * Copyright (C) 2009 - 2010 Harri Pitkänen <hatapitk@iki.fi>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -26,7 +26,7 @@ namespace libvoikko { namespace spellchecker { namespace suggestion {
 class SuggestionGenerator {
 	public:
 		virtual void generate(SuggestionStatus * s) const = 0;
-		virtual ~SuggestionGenerator() {};
+		virtual ~SuggestionGenerator() {}
 };
 
 }}}
