@@ -1,5 +1,5 @@
 /* Libvoikko: Finnish spellchecker and hyphenator library
- * Copyright (C) 2009 Harri Pitkänen <hatapitk@iki.fi>
+ * Copyright (C) 2009 - 2010 Harri Pitkänen <hatapitk@iki.fi>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -20,13 +20,16 @@
 #define VOIKKO_SPELLCHECKER_SUGGESTION_SUGGESTION_GENERATOR_SPLIT_WORD_H
 
 #include "spellchecker/suggestion/SuggestionGenerator.hpp"
+#include "morphology/Analyzer.hpp"
 
 namespace libvoikko { namespace spellchecker { namespace suggestion {
 
 class SuggestionGeneratorSplitWord : public SuggestionGenerator {
 	public:
+		SuggestionGeneratorSplitWord(morphology::Analyzer * morAnalyzer);
 		void generate(SuggestionStatus * s) const;
-
+	private:
+		morphology::Analyzer * morAnalyzer;
 };
 
 }}}
