@@ -1,5 +1,5 @@
 /* Libvoikko: Finnish spellchecker and hyphenator library
- * Copyright (C) 2009 Harri Pitkänen <hatapitk@iki.fi>
+ * Copyright (C) 2009 - 2010 Harri Pitkänen <hatapitk@iki.fi>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -21,12 +21,13 @@
 
 #include "spellchecker/suggestion/SuggestionStrategy.hpp"
 #include "spellchecker/suggestion/SuggestionGenerator.hpp"
+#include "morphology/Analyzer.hpp"
 
 namespace libvoikko { namespace spellchecker { namespace suggestion {
 
 class SuggestionStrategyOcr : public SuggestionStrategy {
 	public:
-		SuggestionStrategyOcr();
+		SuggestionStrategyOcr(morphology::Analyzer * morAnalyzer, size_t maxCost);
 };
 
 }}}
