@@ -1,5 +1,5 @@
 /* Libvoikko: Library of Finnish language tools
- * Copyright (C) 2009 Harri Pitkänen <hatapitk@iki.fi>
+ * Copyright (C) 2009 - 2010 Harri Pitkänen <hatapitk@iki.fi>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,17 +17,37 @@
  *********************************************************************************/
 
 #include "utils/StringUtils.hpp"
-#include "utils/utils.hpp"
+#include "utf8/utf8.hpp"
 #include <cstring>
 #include <cstdlib>
-#include "wchar.h"
+#include <cwchar>
 
 using namespace std;
 
 namespace libvoikko { namespace utils {
 
 wchar_t * StringUtils::ucs4FromUtf8(const char * const original) {
-	return voikko_cstrtoucs4(original, "UTF-8", strlen(original));
+	return ucs4FromUtf8(original, strlen(original));
+}
+	
+wchar_t * StringUtils::ucs4FromUtf8(const char * const original, size_t byteCount) {
+	wchar_t * ucs4Buffer = 0;
+	try {
+		size_t chars = utf8::distance(original, original + byteCount);
+		ucs4Buffer = new wchar_t[chars + 1];
+		const char * origPtr = original;
+		for (size_t i = 0; i < chars; i++) {
+			// Using unchecked function because validity was already
+			// checked with utf8::distance.
+			ucs4Buffer[i] = utf8::unchecked::next(origPtr);
+		}
+		ucs4Buffer[chars] = L'\0';
+		return ucs4Buffer;
+	} catch (...) {
+		// invalid UTF-8 sequence or not enough memory
+		delete[] ucs4Buffer;
+		return 0;
+	}
 }
 
 char * StringUtils::utf8FromUcs4(const wchar_t * const original) {
@@ -35,7 +55,25 @@ char * StringUtils::utf8FromUcs4(const wchar_t * const original) {
 }
 
 char * StringUtils::utf8FromUcs4(const wchar_t * const original, size_t wlen) {
-	return voikko_ucs4tocstr(original, "UTF-8", wlen);
+	char * utf8Buffer;
+	try {
+		utf8Buffer = new char[wlen * 6 + 1];
+	} catch (...) {
+		// not enough memory
+		return 0;
+	}
+	try {
+		char * utfPtr = utf8Buffer;
+		for (size_t i = 0; i < wlen; i++) {
+			utfPtr = utf8::append(original[i], utfPtr);
+		}
+		*utfPtr = '\0';
+		return utf8Buffer;
+	} catch (...) {
+		// invalid codepoint
+		delete[] utf8Buffer;
+		return 0;
+	}
 }
 
 wchar_t * StringUtils::copy(const wchar_t * const original) {
@@ -114,4 +152,37 @@ wchar_t * StringUtils::stripSpecialCharsForMalaga(wchar_t * & original, size_t o
 	return wordBuffer;
 }
 
+bool StringUtils::isInteger(const wchar_t * word) {
+	for (size_t i = 0; word[i] != L'\0'; i++) {
+		if (word[i] < 0x30 || word[i] > 0x39) {
+			return false;
+		}
+	}
+	return true;
+}
+
+bool StringUtils::isRomanNumeral(const wchar_t * word) {
+	// TODO: does not handle all possibilities, does not validate
+	for (size_t i = 0; word[i] != L'\0'; i++) {
+		if (!wcschr(L"iIvVxX", word[i])) {
+			return false;
+		}
+	}
+	return true;
+}
+
+
+bool StringUtils::isPossibleListItem(const wchar_t * word) {
+	if (wcslen(word) == 1) {
+		return true;
+	}
+	if (isInteger(word)) {
+		return true;
+	}
+	if (isRomanNumeral(word)) {
+		return true;
+	}
+	return false;
+}
+
 } }
